@@ -6,6 +6,8 @@
 #ifndef KUNIFIEDPUSH_ABSTRACTPUSHPROVIDER_H
 #define KUNIFIEDPUSH_ABSTRACTPUSHPROVIDER_H
 
+#include "../shared/urgency_p.h"
+
 #include <QObject>
 
 class QNetworkAccessManager;
@@ -30,6 +32,7 @@ public:
         NoError, ///< operation succeeded
         ProviderRejected, ///< communication worked, but the provider refused to complete the operation
         TransientNetworkError, ///< temporary network error, try again
+        ActionRequired, ///< user interaction is required to resolve the problem, e.g. no push server set up
     };
     Q_ENUM(Error)
 
@@ -39,8 +42,14 @@ public:
      */
     virtual bool loadSettings(const QSettings &settings) = 0;
 
+    /** Reset any internal state for a fresh setup connecting to
+     *  a different push server instance.
+     *  The default implementation does nothing.
+     */
+    virtual void resetSettings(QSettings &settings);
+
     /** Attempt to establish a connection to the push provider. */
-    virtual void connectToProvider() = 0;
+    virtual void connectToProvider(Urgency urgency) = 0;
 
     /** Disconnect and existing connection to the push provider. */
     virtual void disconnectFromProvider() = 0;
@@ -51,8 +60,23 @@ public:
     /** Unregister a client from the provider. */
     virtual void unregisterClient(const Client &client) = 0;
 
+    /** Acknowledge a message.
+     *  The default implementation does nothing apart from indicating successful completion.
+     */
+    virtual void acknowledgeMessage(const Client &client, const QString &messageIdentifier);
+
+    /** Change urgency level as needed.
+     *  Reimplement doChangeUrgency if your provider does this as a separate command.
+     */
+    void changeUrgency(Urgency urgency);
+
     /** Provider id used e.g. in settings. */
-    const char* providerId() const;
+    [[nodiscard]] QLatin1StringView providerId() const;
+
+    /** The urgency level currently used by this provider.
+     *  This might not yet be the requested one if changing that is an asynchronous operation.
+     */
+    [[nodiscard]] Urgency urgency() const;
 
 Q_SIGNALS:
     /** Inform about a received push notification. */
@@ -70,13 +94,26 @@ Q_SIGNALS:
     /** Emitted after the connection to the push provider disconnected or failed to be established. */
     void disconnected(KUnifiedPush::AbstractPushProvider::Error error, const QString &errorMsg = {});
 
+    /** Emitted after a message reception has been acknowledge to the push server. */
+    void messageAcknowledged(const KUnifiedPush::Client &client, const QString &messageIdentifier);
+
+    /** Emitted when the urgency level change request has been executed. */
+    void urgencyChanged();
+
 protected:
-    explicit AbstractPushProvider(const char *providerId, QObject *parent);
+    /** Re-implement if urgency leve changes are done as a separate command.
+     *  The default implementation assumes urgency levels aren't supported by this provider.
+     */
+    virtual void doChangeUrgency(Urgency urgency);
+    inline void setUrgency(Urgency urgency) { m_currentUrgency = urgency; }
+
+    explicit AbstractPushProvider(QLatin1StringView providerId, QObject *parent);
     QNetworkAccessManager *nam();
 
 private:
-    const char *const m_providerId;
+    const QLatin1StringView m_providerId;
     QNetworkAccessManager *m_nam = nullptr;
+    Urgency m_currentUrgency = AllUrgencies;
 };
 
 }
